import { ChildModel, DefineModel, ModelField } from '../src/decorators';
import { App } from 'koishi';
import { registerModel } from '../src/register';
import MemoryDatabase from '@koishijs/plugin-database-memory';

declare module 'koishi' {
  // eslint-disable-next-line @typescript-eslint/no-empty-interface
  interface User {
    dress: Dress;
    shirt: Shirt;
  }
}

class Dress {
  @ModelField('string(8)')
  color: string;
  @ModelField('integer(7)')
  size: string;
}

class Shirt {
  @ModelField('string(8)')
  color: string;
  @ModelField('integer(7)')
  size: string;
}

@DefineModel('user')
class UserMixin1 {
  @ChildModel()
  dress: Dress;
}

@DefineModel('user')
class UserMixin2 {
  @ChildModel()
  shirt: Shirt;
}

describe('Model test', () => {
  let app: App;

  beforeEach(async () => {
    app = new App();
    app.plugin(MemoryDatabase);
    await app.start();
  });

  it('should register model fields', () => {
    registerModel(app, UserMixin1);
    registerModel(app, UserMixin2);
    const { user } = app.model.config;
    expect(user.fields['dress.color'].type).toBe('string');
    expect(user.fields['dress.size'].type).toBe('integer');
    expect(user.fields['dress.color'].length).toBe(8);
    expect(user.fields['dress.size'].length).toBe(7);
    expect(user.fields['shirt.color'].type).toBe('string');
    expect(user.fields['shirt.size'].type).toBe('integer');
    expect(user.fields['shirt.color'].length).toBe(8);
    expect(user.fields['shirt.size'].length).toBe(7);
    expect(user.internal['dress.']).toEqual(Dress.prototype);
    expect(user.internal['shirt.']).toEqual(Shirt.prototype);
  });
});
